@@ -50,7 +50,7 @@ setGeneric("extraColumnSlotNames",
 
 setMethod("extraColumnSlotNames", "ANY", function(x) character())
 
-setMethod("fixedColumnNames", "GenomicRanges", function(x) {
+setMethod("parallelVectorNames", "GenomicRanges", function(x) {
   colnames(as.data.frame(new(class(x))))
 })
 
@@ -224,41 +224,30 @@ setValidity2("GenomicRanges", .valid.GenomicRanges)
 ### Coercion.
 ###
 
-setAs("GenomicRanges", "RangedData",
-    function(from)
+### Propagate the names.
+setMethod("as.character", "GenomicRanges",
+    function(x, ignore.strand=FALSE)
     {
-        mcols <- mcols(from)
-        ecs <- extraColumnSlotsAsDF(from)
-        if (length(ecs))
-          mcols <- cbind(mcols, ecs)
-        rd <- RangedData(ranges(from), strand=strand(from),
-                         mcols, space=seqnames(from))
-        mcols(ranges(rd)) <- DataFrame(seqlengths=seqlengths(from),
-                                       isCircular=isCircular(from),
-                                       genome=genome(from))
-        metadata(ranges(rd)) <- metadata(from)
-        metadata(ranges(rd))$seqinfo <- seqinfo(from)
-        rd
+        if (!isTRUEorFALSE(ignore.strand))
+            stop(wmsg("'ignore.strand' must be TRUE or FALSE"))
+        if (length(x) == 0L)
+            return(setNames(character(0), names(x)))
+        ans <- paste0(seqnames(x), ":", start(x), "-", end(x))
+        names(ans) <- names(x)
+        if (ignore.strand)
+            return(ans)
+        x_strand <- strand(x)
+        if (all(x_strand == "*"))
+            return(ans)
+        setNames(paste0(ans, ":", x_strand), names(x))
     }
 )
 
-setAs("GenomicRanges", "RangesList",
-    function(from)
-    {
-        strand_mcols <- DataFrame(strand=strand(from), mcols(from))
-        ecs <- extraColumnSlotsAsDF(from)
-        if (length(ecs))
-          strand_mcols <- cbind(strand_mcols, ecs)
-        rngs <- ranges(from)
-        mcols(rngs) <- strand_mcols
-        rl <- split(rngs, seqnames(from))
-        mcols(rl) <- DataFrame(seqlengths=seqlengths(from),
-                               isCircular=isCircular(from),
-                               genome=genome(from))
-        metadata(rl) <- metadata(from)
-        metadata(rl)$seqinfo <- seqinfo(from)
-        rl
-    }
+### The as.factor() generic doesn't have the ... argument so this method
+### cannot support the 'ignore.strand' argument.
+setMethod("as.factor", "GenomicRanges",
+    function(x)
+        factor(as.character(x), levels=as.character(sort(unique(x))))
 )
 
 ### TODO: Turn this into an S3/S4 combo for as.data.frame.GenomicRanges
@@ -285,6 +274,43 @@ setMethod("as.data.frame", "GenomicRanges",
     }
 )
 
+setAs("GenomicRanges", "RangesList",
+    function(from)
+    {
+        strand_mcols <- DataFrame(strand=strand(from), mcols(from))
+        ecs <- extraColumnSlotsAsDF(from)
+        if (length(ecs))
+          strand_mcols <- cbind(strand_mcols, ecs)
+        rngs <- ranges(from)
+        mcols(rngs) <- strand_mcols
+        rl <- split(rngs, seqnames(from))
+        mcols(rl) <- DataFrame(seqlengths=seqlengths(from),
+                               isCircular=isCircular(from),
+                               genome=genome(from))
+        metadata(rl) <- metadata(from)
+        metadata(rl)$seqinfo <- seqinfo(from)
+        rl
+    }
+)
+
+setAs("GenomicRanges", "RangedData",
+    function(from)
+    {
+        mcols <- mcols(from)
+        ecs <- extraColumnSlotsAsDF(from)
+        if (length(ecs))
+          mcols <- cbind(mcols, ecs)
+        rd <- RangedData(ranges(from), strand=strand(from),
+                         mcols, space=seqnames(from))
+        mcols(ranges(rd)) <- DataFrame(seqlengths=seqlengths(from),
+                                       isCircular=isCircular(from),
+                                       genome=genome(from))
+        metadata(ranges(rd)) <- metadata(from)
+        metadata(ranges(rd))$seqinfo <- seqinfo(from)
+        rd
+    }
+)
+
 .fromSeqinfoToGRanges <- function(from)
 {
     if (any(is.na(seqlengths(from))))
@@ -475,7 +501,7 @@ setMethod("clone", "ANY",  # not exported
 setMethod("update", "GenomicRanges",
           function(object, ...)
           {
-            BiocGenerics:::updateS4(object, ...)
+            BiocGenerics:::replaceSlots(object, ...)
           })
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -628,20 +654,48 @@ setReplaceMethod("[", "GenomicRanges",
     }
 )
 
-### Subsetting a named list-like object *by* a GenomicRanges subscript.
-### The returned object has the class of 'x' and is parallel to 'gr'.
+### Subset a named list-like object *by* a GenomicRanges subscript.
+### The returned object 'ans' is as follow:
+###   (a) 'ans' is parallel to 'gr'.
+###   (b) 'names(ans)' is identical to 'as.character(seqnames(gr))'.
+###   (c) 'elementLengths(ans)' is the same as 'width(gr)'.
+###   (d) 'class(ans)' is 'relistToClass(x[[1]])' e.g. CompressedRleList if
+###       'x' is an RleList object, or DNAStringSet is 'x' is a DNAStringSet
+###       object.
 .subset_by_GenomicRanges <- function(x, gr)
 {
+    if (!(is.list(x) || is(x, "List")))
+        stop(wmsg("'x' must be a list-like object when subsetting ",
+                  "by a GenomicRanges subscript"))
     x_names <- names(x)
-    if (!(is.list(x) || is(x, "List")) || is.null(x_names))
-        stop("subsetting by a GenomicRanges subscript only works ",
-             "on a list-like object with names")
+    if (is.null(x_names))
+        stop(wmsg("'x' must have names when subsetting ",
+                  "by a GenomicRanges subscript"))
+    if (anyDuplicated(x_names))
+        stop(wmsg("'x' must have unique names when subsetting ",
+                  "by a GenomicRanges subscript"))
     irl <- split(ranges(gr), seqnames(gr), drop=TRUE)
     seqlevels_in_use <- names(irl)
     seqlevels2names <- match(seqlevels_in_use, x_names)
     if (any(is.na(seqlevels2names)))
-        stop("subsetting by a GenomicRanges subscript, the names of the ",
-             "object to subset must contain the seqnames of the subscript")
+        stop(wmsg("when subsetting by a GenomicRanges subscript, the names ",
+                  "of the object to subset must contain the seqnames of the ",
+                  "subscript"))
+
+    ## Handle empty case.
+    if (length(gr) == 0L) {
+        if (length(x) != 0L) {
+            x1 <- x[[1L]]
+        } else if (is(x, "CompressedList")) {
+            x1 <- unlist(x, use.names=FALSE)
+        } else {
+            x1 <- new(elementType(x))
+        }
+        unlisted_ans <- x1[0]
+        ans_partitioning <- PartitioningByEnd(names=character(0))
+        return(relist(unlisted_ans, ans_partitioning))
+    }
+
     tmp <- lapply(seq_along(seqlevels_in_use),
                   function(i) {
                       seqlevel <- seqlevels_in_use[i]
@@ -650,7 +704,7 @@ setReplaceMethod("[", "GenomicRanges",
                   })
 
     ## Unsplit 'tmp'.
-    ans <- as(do.call(c, tmp), class(x))
+    ans <- do.call(c, tmp)
     ans_len <- length(gr)
     idx <- unlist(split(seq_len(ans_len), seqnames(gr), drop=TRUE))
     revidx <- integer(ans_len)
