@@ -85,22 +85,6 @@ setMethod("compare", c("GenomicRanges", "GenomicRanges"),
 )
 
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Element-wise (aka "parallel") comparison of 2 GenomicRanges objects.
-###
-### We only need to implement "==" and "<=" methods. The other comparison
-### binary operators (!=, >=, <, >) will then work out-of-the-box on
-### GenomicRanges objects thanks to the methods for Vector objects.
-###
-
-setMethod("==", signature(e1="GenomicRanges", e2="GenomicRanges"),
-    function(e1, e2) { .GenomicRanges.compare(e1, e2) == 0L }
-)
-
-setMethod("<=", signature(e1="GenomicRanges", e2="GenomicRanges"),
-    function(e1, e2) { .GenomicRanges.compare(e1, e2) <= 0L }
-)
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### duplicated()
 ###
@@ -114,10 +98,10 @@ setMethod("<=", signature(e1="GenomicRanges", e2="GenomicRanges"),
     if (!identical(incomparables, FALSE))
         stop("\"duplicated\" method for GenomicRanges objects ",
              "only accepts 'incomparables=FALSE'")
-    IRanges:::duplicatedIntegerQuads(as.factor(seqnames(x)),
-                                     as.factor(strand(x)),
-                                     start(x), width(x),
-                                     fromLast=fromLast, method=method)
+    S4Vectors:::duplicatedIntegerQuads(as.factor(seqnames(x)),
+                                       as.factor(strand(x)),
+                                       start(x), width(x),
+                                       fromLast=fromLast, method=method)
 }
 ### S3/S4 combo for duplicated.GenomicRanges
 duplicated.GenomicRanges <- function(x, incomparables=FALSE, ...)
@@ -135,7 +119,7 @@ setMethod("duplicated", "GenomicRanges", .duplicated.GenomicRanges)
 setMethod("match", c("GenomicRanges", "GenomicRanges"),
     function(x, table, nomatch=NA_integer_, incomparables=NULL,
                        method=c("auto", "quick", "hash"),
-                       ignore.strand=FALSE, match.if.overlap=FALSE)
+                       ignore.strand=FALSE)
     {
         if (!isSingleNumberOrNA(nomatch))
             stop("'nomatch' must be a single number or NA")
@@ -146,24 +130,6 @@ setMethod("match", c("GenomicRanges", "GenomicRanges"),
                  "only accepts 'incomparables=NULL'")
         if (!isTRUEorFALSE(ignore.strand))
             stop("'ignore.strand' must be TRUE or FALSE")
-        if (!isTRUEorFALSE(match.if.overlap))
-            stop("'match.if.overlap' must be TRUE or FALSE")
-        if (match.if.overlap) {
-            msg <- c("  Starting with BioC 2.14, ",
-                     "match() on GRanges objects\n  does not support ",
-                     "the 'match.if.overlap' argument anymore. Please use\n\n",
-                     "    findOverlaps(x, table, select=\"first\", ",
-                     "ignore.strand=ignore.strand)\n\n",
-                     "  if you need to do\n\n",
-                     "    match(x, table, ignore.strand=ignore.strand, ",
-                     "match.if.overlap=TRUE)")
-            .Defunct(msg=msg)
-            ans <- findOverlaps(x, table,
-                                select="first", ignore.strand=ignore.strand)
-            if (!is.na(nomatch) && anyMissing(ans))
-                ans[is.na(ans)] <- nomatch
-            return(ans)
-        }
         ## Calling merge() is the way to check that 'x' and 'table' are based
         ## on the same reference genome.
         merge(seqinfo(x), seqinfo(table))
@@ -178,11 +144,11 @@ setMethod("match", c("GenomicRanges", "GenomicRanges"),
         ## Equivalent to (but faster than):
         ##     findOverlaps(x, table, type="equal", select="first")
         ## except when 'x' and 'table' both contain empty ranges.
-        IRanges:::matchIntegerQuads(x_seqnames, x_strand,
-                                    start(x), width(x),
-                                    as.factor(seqnames(table)), table_strand,
-                                    start(table), width(table),
-                                    nomatch=nomatch, method=method)
+        S4Vectors:::matchIntegerQuads(x_seqnames, x_strand,
+                                      start(x), width(x),
+                                      as.factor(seqnames(table)), table_strand,
+                                      start(table), width(table),
+                                      nomatch=nomatch, method=method)
     }
 )
 
@@ -215,10 +181,10 @@ setMethod("order", "GenomicRanges",
         args <- list(...)
         if (length(args) == 1L) {
             x <- args[[1L]]
-            return(IRanges:::orderIntegerQuads(as.factor(seqnames(x)),
-                                               as.factor(strand(x)),
-                                               start(x), width(x),
-                                               decreasing=decreasing))
+            return(S4Vectors:::orderIntegerQuads(as.factor(seqnames(x)),
+                                                 as.factor(strand(x)),
+                                                 start(x), width(x),
+                                                 decreasing=decreasing))
         }
         order_args <- vector("list", 4L*length(args))
         idx <- 4L*seq_len(length(args))
@@ -236,38 +202,24 @@ setMethod("order", "GenomicRanges",
 ### S3/S4 combo for sort.GenomicRanges
 .sort.GenomicRanges <- function(x, decreasing=FALSE, ignore.strand=FALSE, by)
 {
-    if (!missing(by)) {
-        if (!missing(ignore.strand)) {
-            warning("'ignore.strand' ignored when 'by' is specified")
-        }
-        return(IRanges:::sortBy(x, by, decreasing=decreasing))
-    }
     if (!isTRUEorFALSE(ignore.strand))
         stop("'ignore.strand' must be TRUE or FALSE")
     if (ignore.strand) {
         x2 <- unstrand(x)
         i <- order(x2, decreasing=decreasing)
     } else {
-        i <- order(x, decreasing=decreasing)
+        if (!missing(by)) {
+            if (!missing(ignore.strand)) {
+                warning("'ignore.strand' ignored when 'by' is specified")
+            }
+            i <- S4Vectors:::orderBy(by, x, decreasing=decreasing)
+        } else {
+            i <- order(x, decreasing=decreasing)
+        }
     }
-    IRanges:::extractROWS(x, i)
+    extractROWS(x, i)
 }
 sort.GenomicRanges <- function(x, decreasing=FALSE, ...)
     .sort.GenomicRanges(x, decreasing=decreasing, ...)
 setMethod("sort", "GenomicRanges", .sort.GenomicRanges)
 
-setMethod("rank", "GenomicRanges",
-    function(x, na.last=TRUE,
-             ties.method=c("average", "first", "random", "max", "min"))
-    {
-        if (!missing(ties.method) && !identical(ties.method, "first"))
-            stop("only 'ties.method=\"first\"' is supported ",
-                 "when ranking genomic ranges")
-        oo <- order(x)
-        ## 'ans' is the reverse permutation of 'oo'
-        ans <- integer(length(oo))
-        ans[oo] <- seq_len(length(oo))
-        ans
-    }
-)
-
