@@ -32,6 +32,28 @@ setMethod("clone", "ShallowData",  # not exported
 ##
 ## SummarizedExperiment
 
+## displayed by validity, show, and SummarizedExperiment-to-ExpressionSet
+## coercion methods
+.SummarizedExperiment_deprecation_msg <- wmsg(
+    "The SummarizedExperiment class defined in the GenomicRanges package ",
+    "is deprecated and being replaced with the RangedSummarizedExperiment ",
+    "class defined in the new SummarizedExperiment package. ",
+    "You can use updateObject() on any SummarizedExperiment object ",
+    "to turn it into a RangedSummarizedExperiment."
+)
+
+## displayed by SummarizedExperiment() constructor if SummarizedExperiment
+## package cannot be loaded
+.cannot_load_SummarizedExperiment_msg <- wmsg(
+    "The SummarizedExperiment class defined in the GenomicRanges package ",
+    "is deprecated and being replaced with the RangedSummarizedExperiment ",
+    "class defined in the new SummarizedExperiment package. ",
+    "Please make sure to install the SummarizedExperiment package before ",
+    "you attempt to call the SummarizedExperiment() constructor function. ",
+    "Note that this will return a RangedSummarizedExperiment instance ",
+    "instead of a SummarizedExperiment instance."
+)
+
 setClass("SummarizedExperiment",
     representation(
         exptData="SimpleList",                # overall description
@@ -39,7 +61,7 @@ setClass("SummarizedExperiment",
         colData="DataFrame",                  # columns and their annotations
         assays="Assays"),                     # Data -- e.g., list of matricies
     prototype(
-        rowData=GRanges(),
+        rowData=new("GRanges"),
         assays=.ShallowSimpleListAssays(data=SimpleList())))
 
 ## validity
@@ -85,6 +107,7 @@ setClass("SummarizedExperiment",
 
 .valid.SummarizedExperiment <- function(x)
 {
+    .Deprecated(msg=.SummarizedExperiment_deprecation_msg)
     c(.valid.SummarizedExperiment.assays_current(x),
       msg <- .valid.SummarizedExperiment.assays_class(x),
       if (is.null(msg)) {
@@ -107,68 +130,35 @@ setGeneric("SummarizedExperiment",
 }
 
 setMethod(SummarizedExperiment, "SimpleList",
-   function(assays, rowRanges=GRangesList(), colData=DataFrame(),
-             exptData=SimpleList(), ...,
-             verbose=FALSE)
+   function(assays, ...)
 {
-    extra_args <- list(...)
-    rowData <- extra_args$rowData
-    if (!is.null(rowData)) {
-        if (!missing(rowRanges))
-            stop("both 'rowRanges' and 'rowData' are specified")
-        msg <- c("The 'rowData' argument is deprecated. ",
-                 "Please use 'rowRanges' instead.")
-        .Deprecated(msg=msg)
-        rowRanges <- rowData
-        extra_args$rowData <- NULL
-    } else if (missing(rowRanges) && 0L != length(assays)) {
-        rowRanges <- .GRangesList_assays(assays)
-    }
-    if (missing(colData) && 0L != length(assays)) {
-        nms <- colnames(assays[[1]])
-        if (is.null(nms) && 0L != ncol(assays[[1]]))
-            stop("'SummarizedExperiment' assay colnames must not be NULL")
-        colData <- DataFrame(row.names=nms)
-    }
-
-    FUN <- function(x) {
-        exp <- list(names(rowRanges), rownames(colData))
-        ## dimnames as NULL or list(NULL, NULL)
-        all(sapply(dimnames(x), is.null)) ||
-            ## or consistent with row / colData
-            identical(dimnames(x)[1:2], exp)
-    }
-    if (!all(sapply(assays, FUN)))
-        assays <- endoapply(assays, unname)
-    if (!is(assays, "Assays"))
-        assays <- .ShallowSimpleListAssays(data=assays)
-    do.call(new, c(list("SummarizedExperiment",
-                        exptData=exptData,
-                        rowData=rowRanges,
-                        colData=colData,
-                        assays=assays),
-                   extra_args))
+    if (!requireNamespace("SummarizedExperiment", quietly=TRUE))
+        stop(.cannot_load_SummarizedExperiment_msg)
+    SummarizedExperiment::SummarizedExperiment(assays, ...)
 })
 
 setMethod(SummarizedExperiment, "missing",
     function(assays, ...)
 {
-    SummarizedExperiment(SimpleList(), ...)
+    if (!requireNamespace("SummarizedExperiment", quietly=TRUE))
+        stop(.cannot_load_SummarizedExperiment_msg)
+    SummarizedExperiment::SummarizedExperiment(assays, ...)
 })
 
 setMethod(SummarizedExperiment, "list",
     function(assays, ...)
 {
-    SummarizedExperiment(do.call(SimpleList, assays), ...)
+    if (!requireNamespace("SummarizedExperiment", quietly=TRUE))
+        stop(.cannot_load_SummarizedExperiment_msg)
+    SummarizedExperiment::SummarizedExperiment(assays, ...)
 })
 
 setMethod(SummarizedExperiment, "matrix",
     function(assays, ...)
 {
-    if (is.list(assays))
-        ## special case -- matrix of lists
-        assays <- list(assays)
-    SummarizedExperiment(SimpleList(assays), ...)
+    if (!requireNamespace("SummarizedExperiment", quietly=TRUE))
+        stop(.cannot_load_SummarizedExperiment_msg)
+    SummarizedExperiment::SummarizedExperiment(assays, ...)
 })
 
 ## update / clone
@@ -225,13 +215,13 @@ setGeneric("rowRanges<-",
 
 rowData <- function(...)
 {
-    .Deprecated("rowRanges")
+    .Defunct("rowRanges")
     rowRanges(...)
 }
 
 `rowData<-` <- function(x, ..., value)
 {
-    .Deprecated("rowRanges<-")
+    .Defunct("rowRanges<-")
     rowRanges(x, ...) <- value
 }
 
@@ -458,7 +448,7 @@ setReplaceMethod("dimnames", c("SummarizedExperiment", "NULL"),
     orig <- idx
     idx <- match(idx, txt)
     if (any(bad <- is.na(idx))) {
-        msg <- paste(BiocGenerics:::selectSome(orig[bad]), collapse=" ")
+        msg <- paste(S4Vectors:::selectSome(orig[bad]), collapse=" ")
         stop(sprintf(fmt, msg))
     }
     idx
@@ -809,27 +799,17 @@ setReplaceMethod("$", c("SummarizedExperiment", "ANY"),
     x
 })
 
-## compatibility
-
-setMethod(updateObject, "SummarizedExperiment",
-    function(object, ..., verbose=FALSE)
-{
-    s <- slot(object, "assays")
-    if (is(s, "SimpleList"))
-        slot(object, "assays") <- .ShallowSimpleListAssays(data=s)
-    object
-})
-
 ## show
 
 setMethod(show, "SummarizedExperiment",
     function(object)
 {
-    selectSome <- BiocGenerics:::selectSome
+    .Deprecated(msg=.SummarizedExperiment_deprecation_msg)
+    selectSome <- S4Vectors:::selectSome
     scat <- function(fmt, vals=character(), exdent=2, ...)
     {
         vals <- ifelse(nzchar(vals), vals, "''")
-        lbls <- paste(BiocGenerics:::selectSome(vals), collapse=" ")
+        lbls <- paste(S4Vectors:::selectSome(vals), collapse=" ")
         txt <- sprintf(fmt, length(vals), lbls)
         cat(strwrap(txt, exdent=exdent, ...), sep="\n")
     }
@@ -854,247 +834,63 @@ setMethod(show, "SummarizedExperiment",
     scat("colData names(%d): %s\n", names(colData(object)))
 })
 
-.from_rowRanges_to_FeatureData <- function(from)
-{
-    if (is(from, "GRanges")) {
-        fd <- .from_GRanges_to_FeatureData(from)
-    } else if (is(from, "GRangesList")) {
-        fd <- .from_GRangesList_to_FeatureData(from)
-    } else {
-        stop("class: ", sQuote(class(from)), " is not a supported type for rowRanges coercion")
+## transition to RangedSummarizedExperiment
+
+suppressMessages(
+  setAs("SummarizedExperiment", "RangedSummarizedExperiment",
+    function(from)
+    {
+        if (!requireNamespace("SummarizedExperiment", quietly=TRUE))
+            stop(wmsg("Couldn't load the SummarizedExperiment package! ",
+                      "Please make sure to install the SummarizedExperiment ",
+                      "package before you attempt to coerce a ",
+                      "SummarizedExperiment object to a ",
+                      "RangedSummarizedExperiment object."))
+        SummarizedExperiment:::.from_SummarizedExperiment_to_RangedSummarizedExperiment(from)
     }
-    Biobase::featureNames(fd) <- names(from)
-    fd
-}
+  )
+)
 
-.from_GRanges_to_FeatureData <- function(from)
+setMethod(updateObject, "SummarizedExperiment",
+    function(object, ..., verbose=FALSE)
 {
-    data <- as.data.frame(from)
+    if (!requireNamespace("SummarizedExperiment", quietly=TRUE))
+        stop(wmsg("Couldn't load the SummarizedExperiment package! ",
+                  "Please make sure to install the SummarizedExperiment ",
+                  "package before you attempt to update a ",
+                  "SummarizedExperiment object."))
+    s <- slot(object, "assays")
+    if (is(s, "SimpleList"))
+        slot(object, "assays") <- .ShallowSimpleListAssays(data=s)
+    as(object, "RangedSummarizedExperiment")
+})
 
-    ## the first mcols are automatically included in the data.frame from
-    ## as.data.frame, the secondary mcols holds the metadata for the first
-    ## metadata columns.
-    metaData <- mcols(mcols(from))
-    if (is.null(metaData)) {
-        metaData <- as.data.frame(matrix(ncol=0, nrow=NCOL(data)))
-    } else {
-        metaData <- as.data.frame(metaData)
-    }
-    Biobase::AnnotatedDataFrame(data, metaData)
-}
-.from_GRangesList_to_FeatureData <- function(from)
-{
-    data <- as.data.frame(mcols(from))
-
-    ## the first mcols are automatically included in the data.frame from
-    ## as.data.frame, the secondary mcols holds the metadata for the first
-    ## metadata columns.
-    metaData <- mcols(mcols(from))
-    if (is.null(metaData)) {
-        metaData <- as.data.frame(matrix(ncol=0, nrow=NCOL(data)))
-    } else {
-        metaData <- as.data.frame(metaData)
-    }
-    Biobase::AnnotatedDataFrame(data, metaData)
-}
-
-.from_AnnotatedDataFrame_to_DataFrame <- function(from)
-{
-    df <- DataFrame(Biobase::pData(from))
-    mcols(df) <- DataFrame(Biobase::varMetadata(from))
-    df
-}
-
-.from_DataFrame_to_AnnotatedDataFrame <- function(df)
-{
-    data <- as(df, "data.frame")
-    metaData <- mcols(df)
-    if (is.null(metaData)) {
-        metaData <- as.data.frame(matrix(ncol=0, nrow=NCOL(data)))
-    } else {
-        metaData <- as(metaData, "data.frame")
-    }
-    Biobase::AnnotatedDataFrame(data, metaData)
-}
-
-# If the ExpressionSet has featureData with range information make GRanges out
-# of that, otherwise make an empty GRangesList with names from the featureNames
-naiveRangeMapper <- function(from)
-{
-    nms <- Biobase::featureNames(from)
-    res <- tryCatch(makeGRangesFromDataFrame(Biobase::pData(Biobase::featureData(from)), keep.extra.columns = TRUE),
-             error = function(e) {
-                 res <- relist(GRanges(), vector("list", length=length(nms)))
-                 mcols(res) <- .from_AnnotatedDataFrame_to_DataFrame(Biobase::featureData(from))
-                 res
-             })
-    names(res) <- nms
-    res
-}
-
-# Simple ProbeId to Range mapper
-# Probes with multiple ranges are dropped
-# The sign of the chromosome location is assumed to contain the strand
-# information
-probeRangeMapper <- function(from)
-{
-    annotation <- Biobase::annotation(from)
-    if (identical(annotation, character(0))) {
-        return(naiveRangeMapper(from))
-    }
-    if (requireNamespace("annotate", quietly = TRUE)) {
-        annotationPackage <- annotate::annPkgName(annotation)
-        if (require(annotationPackage, character.only = TRUE, quietly = TRUE)) {
-            db <- get(annotationPackage, envir = asNamespace(annotationPackage))
-            pid <- Biobase::featureNames(from)
-            locs <- AnnotationDbi::select(db, pid, columns = c("CHR", "CHRLOC", "CHRLOCEND"))
-            locs <- na.omit(locs)
-            dups <- duplicated(locs$PROBEID)
-            if (any(dups)) {
-                locs <- locs[!dups, , drop = FALSE]
-            }
-            strand <- ifelse(locs$CHRLOC > 0, "+", "-")
-            res <- GRanges(seqnames = locs$CHR,
-                           ranges = IRanges(abs(locs$CHRLOC), abs(locs$CHRLOCEND)),
-                           strand = strand)
-            names(res) <- locs$PROBEID
-
-            if (NROW(res) < length(pid)) {
-                warning(length(pid) - NROW(res), " probes could not be mapped.", call. = FALSE)
-            }
-            res
-        } else {
-            stop("Failed to load ", sQuote(annotationPackage), " package", call. = FALSE)
-        }
-    } else {
-        stop("Failed to load annotate package", call. = FALSE)
-    }
-}
-
-# Simple ProbeId to Gene mapper
-# Is there a way to get the txDb given the annotation package?
-geneRangeMapper <- function(txDbPackage, key = "ENTREZID")
-{
-    function(from) {
-        annotation <- Biobase::annotation(from)
-        if (identical(annotation, character(0))) {
-            return(naiveRangeMapper(from))
-        }
-        if (requireNamespace("annotate", quietly = TRUE)) {
-            annotationPackage <- annotate::annPkgName(annotation)
-            if (require(annotationPackage, character.only = TRUE, quietly = TRUE)) {
-                db <- get(annotationPackage, envir = asNamespace(annotationPackage))
-                pid <- Biobase::featureNames(from)
-                probeIdToGeneId <- AnnotationDbi::mapIds(db, pid, key, "PROBEID")
-                geneIdToProbeId <- setNames(names(probeIdToGeneId), probeIdToGeneId)
-
-                if (requireNamespace(txDbPackage, quietly = TRUE)) {
-                    txDb <- get(txDbPackage, envir = asNamespace(txDbPackage))
-                    genes <- GenomicFeatures::genes(txDb)
-                    probesWithAMatch <- probeIdToGeneId[probeIdToGeneId %in% names(genes)]
-                    res <- genes[probesWithAMatch]
-                    names(res) <- geneIdToProbeId[names(res)]
-                    if (NROW(res) < length(pid)) {
-                        warning(length(pid) - NROW(res), " probes could not be mapped.", call. = FALSE)
-                    }
-                    res
-                } else {
-                    stop("Failed to load ", sQuote(txDbPackage), " package", call. = FALSE)
-                }
-
-            } else {
-                stop("Failed to load ", sQuote(annotationPackage), " package", call. = FALSE)
-            }
-        } else {
-            stop("Failed to load annotate package", call. = FALSE)
-        }
-    }
-}
-
-makeSummarizedExperimentFromExpressionSet <- function(from, mapFun = naiveRangeMapper)
-{
-    if (requireNamespace("Biobase", quietly = TRUE)) {
-        mapFun <- match.fun(mapFun)
-        rowRanges <- mapFun(from)
-        matches <- match(names(rowRanges),
-                         Biobase::featureNames(from),
-                         nomatch = 0)
-        from <- from[matches, drop = FALSE]
-        assays <- as.list(Biobase::assayData(from))
-        colData <- .from_AnnotatedDataFrame_to_DataFrame(Biobase::phenoData(from))
-        exptData <- SimpleList(
-                               experimentData = Biobase::experimentData(from),
-                               annotation = Biobase::annotation(from),
-                               protocolData = Biobase::protocolData(from)
-                               )
-
-        SummarizedExperiment(
-                             assays = assays,
-                             rowRanges = rowRanges,
-                             colData = colData,
-                             exptData = exptData
-                             )
-    }
-}
+suppressMessages(
+  setAs("SummarizedExperiment", "ExpressionSet",
+    function(from)
+    {
+        .Deprecated(msg=.SummarizedExperiment_deprecation_msg)
+        as(as(from, "RangedSummarizedExperiment"), "ExpressionSet")
+    })
+)
 
 suppressMessages(
     setAs("ExpressionSet", "SummarizedExperiment", function(from)
     {
-        makeSummarizedExperimentFromExpressionSet(from, naiveRangeMapper)
+        warning(wmsg(
+        "The SummarizedExperiment class defined in the GenomicRanges package ",
+        "is deprecated and being replaced with the RangedSummarizedExperiment ",
+        "class defined in the new SummarizedExperiment package. ",
+        "The coercion method from ExpressionSet to SummarizedExperiment ",
+        "has been modified to return a RangedSummarizedExperiment object."
+        ))
+        if (!requireNamespace("SummarizedExperiment", quietly=TRUE))
+            stop(wmsg("Couldn't load the SummarizedExperiment package! ",
+                      "Please make sure to install the SummarizedExperiment ",
+                      "package before you attempt to coerce an ",
+                      "ExpressionSet object to a ",
+                      "RangedSummarizedExperiment object."))
+        SummarizedExperiment::makeSummarizedExperimentFromExpressionSet(from)
     })
 )
 
-suppressMessages(
-    setAs("SummarizedExperiment", "ExpressionSet", function(from)
-    {
-        if (requireNamespace("Biobase", quietly = TRUE)) {
-
-            assayData <- list2env(as.list(assays(from)))
-
-            numAssays <- length(assayData)
-
-            if(numAssays == 0) {
-                assayData$exprs <- new("matrix")
-            } else if (!"exprs" %in% ls(assayData)) {
-                ## if there isn't an exprs assay we need to pick one as exprs, so
-                ## rename the first element exprs and issue a warning.
-                exprs <- ls(assayData)[[1]]
-                warning("No assay named ", sQuote("exprs"), " found, renaming ",
-                        exprs, " to ", sQuote("exprs"), ".")
-                assayData[["exprs"]] <- assayData[[exprs]]
-                rm(list=exprs, envir=assayData)
-            }
-
-            featureData <- .from_rowRanges_to_FeatureData(rowRanges(from))
-            phenoData <- .from_DataFrame_to_AnnotatedDataFrame(colData(from))
-
-            ed <- exptData(from)
-
-            experimentData <- if (!is.null(ed$experimentData)) {
-                ed$experimentData
-            } else {
-                Biobase::MIAME()
-            }
-
-            annotation <- if (!is.null(ed$annotation)) {
-                ed$annotation
-            } else {
-                character()
-            }
-
-            protocolData <- if (!is.null(ed$protocolData)) {
-                ed$protocolData
-            } else {
-                Biobase::annotatedDataFrameFrom(assayData, byrow=FALSE)
-            }
-
-            Biobase::ExpressionSet(assayData,
-                                   phenoData = phenoData,
-                                   featureData = featureData,
-                                   experimentData = experimentData,
-                                   annotation = annotation,
-                                   protocolData = protocolData
-                                   )
-        }
-    })
-)
