@@ -14,14 +14,14 @@
 setMethod(mcols, "SummarizedExperiment",
     function(x, use.names=FALSE, ...)
 {
-    mcols(rowData(x), use.names=use.names, ...)
+    mcols(rowRanges(x), use.names=use.names, ...)
 })
 
 setReplaceMethod("mcols", "SummarizedExperiment",
     function(x, ..., value)
 {
     clone(x, rowData=local({
-        r <- rowData(x)
+        r <- rowRanges(x)
         mcols(r) <- value
         r
     }))
@@ -33,26 +33,26 @@ setReplaceMethod("mcols", "SummarizedExperiment",
 setMethod(elementMetadata, "SummarizedExperiment",
     function(x, use.names=FALSE, ...)
 {
-    elementMetadata(rowData(x), use.names=use.names, ...)
+    elementMetadata(rowRanges(x), use.names=use.names, ...)
 })
 
 setReplaceMethod("elementMetadata", "SummarizedExperiment",
     function(x, ..., value)
 {
-    elementMetadata(rowData(x), ...) <- value
+    elementMetadata(rowRanges(x), ...) <- value
     x
 })
 
 setMethod(values, "SummarizedExperiment",
     function(x, ...)
 {
-    values(rowData(x), ...)
+    values(rowRanges(x), ...)
 })
 
 setReplaceMethod("values", "SummarizedExperiment",
     function(x, ..., value)
 {
-    values(rowData(x), ...) <- value
+    values(rowRanges(x), ...) <- value
     x
 })
 
@@ -75,10 +75,10 @@ local({
         formals(tmpl) <- formals(generic)
         fmls <- as.list(formals(tmpl))
         fmls[] <- sapply(names(fmls), as.symbol)
-        fmls[[generic@signature]] <- quote(rowData(x))
+        fmls[[generic@signature]] <- quote(rowRanges(x))
         if (.fun %in% endomorphisms)
             body(tmpl) <- substitute({
-                rowData(x) <- do.call(FUN, ARGS)
+                rowRanges(x) <- do.call(FUN, ARGS)
                 x
             }, list(FUN=.fun, ARGS=fmls))
         else
@@ -95,7 +95,7 @@ setMethod("granges", "SummarizedExperiment",
     if (!identical(use.mcols, FALSE))
         stop("\"granges\" method for SummarizedExperiment objects ",
              "does not support the 'use.mcols' argument")
-    rowData(x)
+    rowRanges(x)
 })
 
 ## 2-argument dispatch:
@@ -108,9 +108,9 @@ setMethod("granges", "SummarizedExperiment",
     function(x, y)
 {
     if (is(x, "SummarizedExperiment"))
-        x <- rowData(x)
+        x <- rowRanges(x)
     if (is(y, "SummarizedExperiment"))
-        y <- rowData(y)
+        y <- rowRanges(y)
     compare(x, y)
 }
 
@@ -118,41 +118,42 @@ setMethod("granges", "SummarizedExperiment",
     function(e1, e2)
 {
     if (is(e1, "SummarizedExperiment"))
-        e1 <- rowData(e1)
+        e1 <- rowRanges(e1)
     if (is(e2, "SummarizedExperiment"))
-        e2 <- rowData(e2)
+        e2 <- rowRanges(e2)
     callGeneric(e1=e1, e2=e2)
 }
 
 .SummarizedExperiment.nearest.missing <-
     function(x, subject, select = c("arbitrary", "all"),
-             ignore.strand = FALSE, ...)
+             algorithm = c("nclist", "intervaltree"), ignore.strand = FALSE)
 {
     select <- match.arg(select)
-    x <- rowData(x)
-    nearest(x=x, subject=x, select=select,
-            ignore.strand=ignore.strand, ignoreSelf=TRUE, ...)
+    x <- rowRanges(x)
+    nearest(x=x, select=select,
+            algorithm=match.arg(algorithm), ignore.strand=ignore.strand)
 }
 
 .SummarizedExperiment.distance <-
     function(x, y, ignore.strand = FALSE, ...)
 {
     if (is(x, "SummarizedExperiment"))
-        x <- rowData(x)
+        x <- rowRanges(x)
     if (is(y, "SummarizedExperiment"))
-        y <- rowData(y)
+        y <- rowRanges(y)
     distance(x, y, ignore.strand=ignore.strand, ...)
 }
 
 .SummarizedExperiment.distanceToNearest <-
-    function(x, subject, ignore.strand = FALSE, ...)
+    function(x, subject, algorithm = c("nclist", "intervaltree"),
+             ignore.strand = FALSE, ...)
 {
     if (is(x, "SummarizedExperiment"))
-        x <- rowData(x)
+        x <- rowRanges(x)
     if (is(subject, "SummarizedExperiment"))
-        subject <- rowData(subject)
-    distanceToNearest(x, subject, ignore.strand=ignore.strand,
-                      ...)
+        subject <- rowRanges(subject)
+    distanceToNearest(x, subject, algorithm=match.arg(algorithm),
+                      ignore.strand=ignore.strand, ...)
 }
 
 local({
@@ -162,19 +163,36 @@ local({
         c("SummarizedExperiment", "SummarizedExperiment"))
 
     for (.sig in .signatures) {
-        .funs <- c("nearest", "precede", "follow")
+        .funs <- c("precede", "follow")
         tmpl <- function(x, subject, select = c("arbitrary", "all"),
-                         ignore.strand = FALSE, ...) {}
+                         ignore.strand = FALSE) {}
         environment(tmpl) <- parent.frame(2)
         for (.fun in .funs) {
             body(tmpl) <- substitute({
                 select <- match.arg(select)
                 if (is(x, "SummarizedExperiment"))
-                    x <- rowData(x)
+                    x <- rowRanges(x)
                 if (is(subject, "SummarizedExperiment"))
-                    subject <- rowData(subject)
+                    subject <- rowRanges(subject)
                 FUN(x=x, subject=subject, select=select,
-                    ignore.strand=ignore.strand, ...)
+                    ignore.strand=ignore.strand)
+            }, list(FUN=as.symbol(.fun)))
+            setMethod(.fun, .sig, tmpl)
+        }
+        .funs <- "nearest"
+        tmpl <- function(x, subject, select = c("arbitrary", "all"),
+                         algorithm=c("nclist", "intervaltree"),
+                         ignore.strand = FALSE) {}
+        environment(tmpl) <- parent.frame(2)
+        for (.fun in .funs) {
+            body(tmpl) <- substitute({
+                select <- match.arg(select)
+                if (is(x, "SummarizedExperiment"))
+                    x <- rowRanges(x)
+                if (is(subject, "SummarizedExperiment"))
+                    subject <- rowRanges(subject)
+                FUN(x=x, subject=subject, select=select,
+                    ignore.strand=ignore.strand)
             }, list(FUN=as.symbol(.fun)))
             setMethod(.fun, .sig, tmpl)
         }
@@ -193,14 +211,14 @@ local({
 setReplaceMethod("strand", "SummarizedExperiment",
     function(x, ..., value)
 {
-    strand(rowData(x)) <- value
+    strand(rowRanges(x)) <- value
     x
 })
 
 setReplaceMethod("ranges", "SummarizedExperiment",
     function(x, ..., value)
 {
-    ranges(rowData(x)) <- value
+    ranges(rowRanges(x)) <- value
     x
 })
 
@@ -209,7 +227,7 @@ setReplaceMethod("ranges", "SummarizedExperiment",
 setMethod("order", "SummarizedExperiment",
     function(..., na.last = TRUE, decreasing = FALSE)
 {
-    args <- lapply(list(...), rowData)
+    args <- lapply(list(...), rowRanges)
     do.call("order",
             c(args, list(na.last=na.last, decreasing=decreasing)))
 })
@@ -219,13 +237,13 @@ setMethod("rank", "SummarizedExperiment",
               ties.method = c("average", "first", "random", "max", "min"))
 {
     ties.method <- match.arg(ties.method)
-    rank(rowData(x), na.last=na.last, ties.method=ties.method)
+    rank(rowRanges(x), na.last=na.last, ties.method=ties.method)
 })
 
 setMethod("sort", "SummarizedExperiment",
     function(x, decreasing = FALSE, ...)
 {
-    x[order(rowData(x), decreasing=decreasing),]
+    x[order(rowRanges(x), decreasing=decreasing),]
 })
 
 ## subset
@@ -233,23 +251,8 @@ setMethod("sort", "SummarizedExperiment",
 setMethod("subset", "SummarizedExperiment",
     function(x, subset, select, ...)
 {
-    top_prenv <- IRanges:::top_prenv
-    i <- if (missing(subset))
-        TRUE
-    else {
-        i <- eval(substitute(subset), rowData(x), top_prenv(subset))
-        i <- tryCatch(as.logical(i), error=function(err) {
-            txt <- sprintf("'subset' must be coercible to logical: %s",
-                           conditionMessage(err))
-            stop(paste(strwrap(txt, exdent=2, "\n")))
-        })
-        i & !is.na(i)
-    }
-    j <- if (missing(select))
-        TRUE
-    else {
-        j <- eval(substitute(select), colData(x), top_prenv(select))
-    }
+    i <- S4Vectors:::evalqForSubset(subset, rowRanges(x), ...)
+    j <- S4Vectors:::evalqForSubset(select, colData(x), ...)
     x[i, j]
 })
 
@@ -261,25 +264,20 @@ setMethod(seqinfo, "SummarizedExperiment",
     seqinfo(x@rowData)
 })
 
-setMethod("seqlevelsInUse", "SummarizedExperiment",
-    function(x)
-{
-    seqlevelsInUse(rowData(x))
-})
-
 setReplaceMethod("seqinfo", "SummarizedExperiment",
     function (x, new2old = NULL, force = FALSE, value)
 {
     if (!is(value, "Seqinfo")) 
         stop("the supplied 'seqinfo' must be a Seqinfo object")
     dangling_seqlevels <-
-        getDanglingSeqlevels(x@rowData, new2old = new2old,
-                             force = force, seqlevels(value))
+        GenomeInfoDb:::getDanglingSeqlevels(x@rowData, new2old = new2old,
+                                            force = force, seqlevels(value))
     if (length(dangling_seqlevels) != 0L) 
         x <- x[!(seqnames(x) %in% dangling_seqlevels)]
     x@rowData <-
         update(x@rowData,
-               seqnames = makeNewSeqnames(x, new2old, seqlevels(value)),
+               seqnames = GenomeInfoDb:::makeNewSeqnames(x, new2old,
+                                                         seqlevels(value)),
                seqinfo = value)
     if (is.character(msg <- .valid.SummarizedExperiment(x)))
         stop(msg)
@@ -288,17 +286,17 @@ setReplaceMethod("seqinfo", "SummarizedExperiment",
 
 ## extractROWS, replaceROWS
 
-setMethod(IRanges:::extractROWS, "SummarizedExperiment",
+setMethod("extractROWS", "SummarizedExperiment",
     function(x, i)
 {
-    ridx <- IRanges:::extractROWS(seq_len(nrow(x)), i)
+    ridx <- extractROWS(seq_len(nrow(x)), i)
     x[ridx,]
 })
 
-setMethod(IRanges:::replaceROWS, "SummarizedExperiment",
+setMethod("replaceROWS", "SummarizedExperiment",
     function (x, i, value)
 {
-    ridx <- IRanges:::extractROWS(seq_len(nrow(x)), i)
+    ridx <- extractROWS(seq_len(nrow(x)), i)
     x[ridx,] <- value
     x
 })
