@@ -108,7 +108,8 @@
 .find_GRanges_cols <- function(df_colnames,
                                seqnames.field=c("seqnames", "seqname",
                                                 "chromosome", "chrom",
-                                                "chr", "chromosome_name"),
+                                                "chr", "chromosome_name",
+                                                "seqid"),
                                start.field="start",
                                end.field=c("end", "stop"),
                                strand.field="strand",
@@ -150,7 +151,8 @@ makeGRangesFromDataFrame <- function(df,
                                      seqinfo=NULL,
                                      seqnames.field=c("seqnames", "seqname",
                                                       "chromosome", "chrom",
-                                                      "chr", "chromosome_name"),
+                                                      "chr", "chromosome_name",
+                                                      "seqid"),
                                      start.field="start",
                                      end.field=c("end", "stop"),
                                      strand.field="strand",
@@ -195,7 +197,8 @@ makeGRangesFromDataFrame <- function(df,
     if (is.na(granges_cols[["strand"]]) || ignore.strand) {
         ans_strand <- "*"
     } else {
-        ans_strand <- df[[granges_cols[["strand"]]]]
+        ans_strand <- as.character(df[[granges_cols[["strand"]]]])
+        ans_strand[ans_strand %in% "."] <- "*"
     }
 
     ## Prepare 'ans_mcols'.
