@@ -138,6 +138,11 @@ allCompatibleSeqnamesAndStrand <- function(x, y) {
       compatibleStrand(strand(x), strand(y)))
 }
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### punion()
+###
+
 setMethod("punion", c("GRanges", "GRanges"),
     function(x, y, fill.gap=FALSE, ignore.strand=FALSE, ...)
     {
@@ -189,98 +194,172 @@ setMethod("punion", c("GRanges", "GRangesList"),
     }
 )
 
-setMethod("pintersect", c("GRanges", "GRanges"),
-    function(x, y, resolve.empty=c("none", "max.start", "start.x"),
-             ignore.strand=FALSE, ...)
-    {
-        if (length(x) != length(y)) 
-            stop("'x' and 'y' must have the same length")
-        if (!isTRUEorFALSE(ignore.strand))
-            stop("'ignore.strand' must be TRUE or FALSE")
-        if (ignore.strand)
-            strand(y) <- strand(x)
-        resolve.empty <- match.arg(resolve.empty)
-        mcols(x) <- NULL
-        seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
-        if (!allCompatibleSeqnamesAndStrand(x, y))
-            stop("'x' and 'y' elements must have compatible 'seqnames' ",
-                 "and 'strand' values")
-        ## Update the ranges.
-        ranges(x) <- pintersect(ranges(x), ranges(y),
-                                resolve.empty=resolve.empty)
-        ## Update the strand.
-        ansStrand <- strand(x)
-        resolveStrand <- as(ansStrand == "*", "IRanges")
-        if (length(resolveStrand) > 0L)
-            ansStrand[as.integer(resolveStrand)] <-
-              extractROWS(strand(y), resolveStrand)
-        strand(x) <- ansStrand
-        x
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### pintersect()
+###
+
+### 'x' and 'y' must have the same length. An 'y' of length 1 is accepted and
+### is recycled to the length of 'x'.
+### Return a GRanges object. If 'drop.nohit.ranges' is FALSE (the default) the
+### returned object is parallel to 'x' with the original ranges modified
+### (seqnames, names, and metadata are untouched), so this can be seen as an
+### "intra-range transformation". The original metadata columns are propagated
+### and a new "hit" column added to them to indicate elements in 'x' that
+### intersect with the corresponding element in 'y'. For these elements the
+### range in the returned object is guaranteed to be a subrange of the
+### original ranges. If 'ignore.strand' or 'strict.strand' is TRUE, then the
+### returned GRanges has the same strand as 'x'. Otherwise, for elements in 'x'
+### that are on the "*" strand and hit the corresponding element in 'y', it
+### has the strand of 'y'.
+### If 'drop.nohit.ranges' is TRUE everything is the same except that elements
+### in 'x' that don't intersect with the corresponding element in 'y' are
+### removed from the result (so the result is no more parallel to the input)
+### and no "hit" metadata column is added to it.
+.pintersect_GRanges_GRanges <- function(x, y, drop.nohit.ranges=FALSE,
+                                              ignore.strand=FALSE,
+                                              strict.strand=FALSE)
+{
+    stopifnot(is(x, "GRanges"), is(y, "GRanges"))
+    if (length(y) != length(x)) {
+        if (length(y) != 1L)
+            stop(wmsg("'y' must have the length of 'x' or length 1"))
+        y <- rep.int(y, length(x))
     }
+    if (!isTRUEorFALSE(drop.nohit.ranges))
+        stop(wmsg("'drop.nohit.ranges' must be TRUE or FALSE"))
+    if (!isTRUEorFALSE(ignore.strand))
+        stop(wmsg("'ignore.strand' must be TRUE or FALSE"))
+    if (!isTRUEorFALSE(strict.strand))
+        stop(wmsg("'strict.strand' must be TRUE or FALSE"))
+    if (ignore.strand) {
+        if (strict.strand)
+            warning(wmsg("'strict.strand' is ignored ",
+                         "when 'ignore.strand' is TRUE"))
+        strand(y) <- strand(x)
+    } else if (!strict.strand) {
+        x_strand <- strand(x)
+        idx <- strand(x) == "*" & strand(y) != "*"
+        strand(x)[idx] <- strand(y)[idx]
+        idx <- strand(y) == "*" & strand(x) != "*"
+        strand(y)[idx] <- strand(x)[idx]
+    }
+
+    x_seqlevels <- seqlevels(x)
+    ## Check compatibility of underlying genomes.
+    seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
+    ## Align the seqlevels of 'y' to those of 'x' so that comparing the
+    ## seqnames of the 2 objects thru as.integer(seqnames()) is meaningful).
+    seqlevels(y) <- seqlevels(x)
+    ## Restore original seqlevels on 'x' (this preserves their order).
+    seqlevels(x) <- x_seqlevels
+
+    new_start <- pmax.int(start(x), start(y))
+    new_end <- pmin.int(end(x), end(y))
+
+    is_hit <- as.integer(seqnames(x)) == as.integer(seqnames(y)) &
+              strand(x) == strand(y) & new_end >= new_start - 1L
+
+    if (drop.nohit.ranges) {
+        x <- extractROWS(x, is_hit)
+        new_start <- extractROWS(new_start, is_hit)
+        new_end <- extractROWS(new_end, is_hit)
+        new_names <- names(x)
+    } else {
+        ## For elements in 'x' and 'y' that don't hit each other, we return
+        ## an artificial zero-width intersection that starts on 'start(x)'.
+        nohit_idx <- which(!is_hit)
+        nohit_start <- start(x)[nohit_idx]
+        new_start[nohit_idx] <- nohit_start
+        new_end[nohit_idx] <- nohit_start - 1L
+        new_names <- names(x)
+        mcols(x)$hit <- as.logical(is_hit)
+        if (!(ignore.strand || strict.strand))
+            strand(x)[nohit_idx] <- x_strand[nohit_idx]
+    }
+    ranges(x) <- IRanges(new_start, new_end, names=new_names)
+    x
+}
+
+setMethod("pintersect", c("GRanges", "GRanges"),
+    function(x, y, drop.nohit.ranges=FALSE,
+                   ignore.strand=FALSE, strict.strand=FALSE)
+        .pintersect_GRanges_GRanges(x, y, drop.nohit.ranges=drop.nohit.ranges,
+                                          ignore.strand=ignore.strand,
+                                          strict.strand=strict.strand)
 )
 
-### TODO: Like for "punion", the semantic of the
-### pintersect,GRangesList,GRanges method should simply derive from
-### the semantic of the pintersect,GRanges,GRanges. Then the
-### implementation and documentation will be both much easier to understand.
-### It's hard to guess what's the current semantic of this method is by
-### just looking at the code below, but it doesn't seem to be one of:
-###   (a) for (i in seq_len(length(x)))
-###         x[[i]] <- pintersect(x[[i]], y[rep.int(i, length(x[[i]]))])
-###   (b) for (i in seq_len(length(x)))
-###         x[[i]] <- intersect(x[[i]], y[i])
-### It seems to be close to (b) but with special treatment of the "*"
-### strand value in 'y'.
-### FIXME: 'resolve.empty' is silently ignored.
-setMethod("pintersect", c("GRangesList", "GRanges"),
-    function(x, y, resolve.empty=c("none", "max.start", "start.x"),
-             ignore.strand=FALSE, ...)
-    {
-        ## TODO: Use "seqinfo<-" method for GRangesList objects when it
-        ## becomes available.
-        seqinfo(x@unlistData) <- merge(seqinfo(x), seqinfo(y))
-        if (length(x) != length(y)) 
-            stop("'x' and 'y' must have the same length")
-        ok <- compatibleSeqnames(seqnames(x@unlistData),
-                                 rep(seqnames(y), elementLengths(x)))
-        if (!ignore.strand)
-          ok <- ok & compatibleStrand(strand(x@unlistData),
-                                      rep(strand(y), elementLengths(x)))
-        ok <-
-          new2("CompressedLogicalList", unlistData=as.vector(ok),
-               partitioning=x@partitioning)
-        if (ncol(mcols(x@unlistData)) > 0L)
-            mcols(x@unlistData) <- NULL
-        if (ncol(mcols(y)) > 0L)
-            mcols(y) <- NULL
-        x <- x[ok]
-        y <- rep(y, sum(ok))
-        x@unlistData@ranges <-
-          pintersect(x@unlistData@ranges, y@ranges, resolve.empty="start.x")
-        x[width(x) > 0L]
+### This is equivalent to 'mendoapply(pintersect, x, y)'. Therefore the
+### returned GRangesList object has the same shape as 'x'.
+### To get the 'mendoapply(intersect, x, y)' behavior, the user should use
+### 'strict.strand=TRUE' and call 'reduce( , drop.empty.ranges=TRUE)' on
+### the returned object.
+.pintersect_GRangesList_GRanges <- function(x, y, drop.nohit.ranges=FALSE,
+                                                  ignore.strand=FALSE,
+                                                  strict.strand=FALSE)
+{
+    stopifnot(is(x, "GRangesList"), is(y, "GRanges"))
+    if (length(y) != length(x)) {
+        if (length(y) != 1L)
+            stop(wmsg("'y' must have the length of 'x' or length 1"))
+        y <- rep.int(y, length(x))
     }
+    if (!isTRUEorFALSE(drop.nohit.ranges))
+        stop(wmsg("'drop.nohit.ranges' must be TRUE or FALSE"))
+    unlisted_x <- unlist(x, use.names=FALSE)
+    y2 <- rep.int(y, elementLengths(x))
+    ## 'unlisted_ans' parallel to 'x'.
+    unlisted_ans <- .pintersect_GRanges_GRanges(unlisted_x, y2,
+                                        drop.nohit.ranges=FALSE,
+                                        ignore.strand=ignore.strand,
+                                        strict.strand=strict.strand)
+    if (drop.nohit.ranges) {
+        is_hit <- relist(mcols(unlisted_ans)$hit, x)
+        mcols(unlisted_ans)$hit <- NULL
+        ans <- relist(unlisted_ans, x)[is_hit]
+    } else {
+        ans <- relist(unlisted_ans, x)
+    }
+    ans
+}
+
+setMethod("pintersect", c("GRangesList", "GRanges"),
+    function(x, y, drop.nohit.ranges=FALSE,
+                   ignore.strand=FALSE, strict.strand=FALSE)
+        .pintersect_GRangesList_GRanges(x, y,
+                                        drop.nohit.ranges=drop.nohit.ranges,
+                                        ignore.strand=ignore.strand,
+                                        strict.strand=strict.strand)
 )
 
 setMethod("pintersect", c("GRanges", "GRangesList"),
-    function(x, y, resolve.empty=c("none", "max.start", "start.x"), ...)
+    function(x, y, drop.nohit.ranges=FALSE,
+                   ignore.strand=FALSE, strict.strand=FALSE)
+        .pintersect_GRangesList_GRanges(y, x,
+                                        drop.nohit.ranges=drop.nohit.ranges,
+                                        ignore.strand=ignore.strand,
+                                        strict.strand=strict.strand)
+)
+
+### Equivalent to 'mendoapply(intersect, x, y)'.
+setMethod("pintersect", c("GRangesList", "GRangesList"),
+    function(x, y, ...)
     {
-        callGeneric(y, x)
+        if (length(x) != length(y))
+            stop("'x' and 'y' must have the same length")
+        seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
+        seqlevels(y) <- seqlevels(x)
+        xgr <- deconstructGRLintoGR(x)
+        ygr <- deconstructGRLintoGR(y)
+        gr <- intersect(xgr, ygr, ...)
+        reconstructGRLfromGR(gr, x)
     }
 )
 
-setMethod("pintersect", c("GRangesList", "GRangesList"),
-          function(x, y, ...)
-          {
-            if (length(x) != length(y))
-              stop("'x' and 'y' must have the same length")
-            seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
-            seqlevels(y) <- seqlevels(x)
-            xgr <- deconstructGRLintoGR(x)
-            ygr <- deconstructGRLintoGR(y)
-            gr <- intersect(xgr, ygr, ...)
-            reconstructGRLfromGR(gr, x)
-          }
-          )
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### psetdiff()
+###
 
 setMethod("psetdiff", c("GRanges", "GRanges"),
     function(x, y, ignore.strand=FALSE, ...)
