@@ -1,4 +1,5 @@
 ### R code from vignette source 'GenomicRangesHOWTOs.Rnw'
+### Encoding: UTF-8
 
 ###################################################
 ### code chunk number 1: style
@@ -665,54 +666,36 @@ cs_by_chrom
 
 
 ###################################################
-### code chunk number 84: bim_1
+### code chunk number 84: bin_1
 ###################################################
 library(BSgenome.Scerevisiae.UCSC.sacCer2)
-set.seed(22)
-cov <- RleList(
+set.seed(55)
+my_var <- RleList(
     lapply(seqlengths(Scerevisiae),
-           function(len) Rle(sample(-10:10, len, replace=TRUE))),
+        function(seqlen) {
+            tmp <- sample(50L, seqlen, replace=TRUE) %/% 50L
+            Rle(cumsum(tmp - rev(tmp)))
+        }
+    ),
     compress=FALSE)
-head(cov, 3)
+my_var
 
 
 ###################################################
 ### code chunk number 85: bin_2
 ###################################################
-bins1 <- tileGenome(seqinfo(Scerevisiae), tilewidth=100,
-                    cut.last.tile.in.chrom=TRUE)
+bins <- tileGenome(seqinfo(Scerevisiae), tilewidth=100,
+                   cut.last.tile.in.chrom=TRUE)
 
 
 ###################################################
 ### code chunk number 86: bin_3
 ###################################################
-binnedAverage <- function(bins, numvar, mcolname)
-{
-    stopifnot(is(bins, "GRanges"))
-    stopifnot(is(numvar, "RleList"))
-    stopifnot(identical(seqlevels(bins), names(numvar)))
-    bins_per_chrom <- split(ranges(bins), seqnames(bins))
-    means_list <- lapply(names(numvar),
-        function(seqname) {
-            views <- Views(numvar[[seqname]],
-                           bins_per_chrom[[seqname]])
-            viewMeans(views)
-        })
-    new_mcol <- unsplit(means_list, as.factor(seqnames(bins)))
-    mcols(bins)[[mcolname]] <- new_mcol
-    bins
-}
+binnedAverage(bins, my_var, "binned_var")
 
 
 ###################################################
-### code chunk number 87: bin_4
-###################################################
-bins1 <- binnedAverage(bins1, cov, "binned_cov")
-bins1
-
-
-###################################################
-### code chunk number 88: SessionInfo
+### code chunk number 87: SessionInfo
 ###################################################
 sessionInfo()
 
