@@ -1,19 +1,18 @@
 make_test_GRangesList <- function() {
     GRangesList(
-        a =
-        new("GRanges",
+        a = GRanges(
             seqnames = Rle(factor(c("chr1", "chr2", "chr1", "chr3")), c(1, 3, 2, 4)),
             ranges = IRanges(1:10, width = 10:1, names = head(letters, 10)),
             strand = Rle(strand(c("-", "+", "*", "+", "-")), c(1, 2, 2, 3, 2)),
             seqinfo = Seqinfo(seqnames = paste("chr", 1:3, sep="")),
-            elementMetadata = DataFrame(score = 1:10, GC = seq(1, 0, length=10))),
-        b =
-        new("GRanges",
+            score = 1:10, GC = seq(1, 0, length=10)),
+        b = GRanges(
             seqnames = Rle(factor(c("chr2", "chr4", "chr5")), c(3, 6, 4)),
             ranges = IRanges(1:13, width = 13:1, names = tail(letters, 13)),
             strand = Rle(strand(c("-", "+", "-")), c(4, 5, 4)),
             seqinfo = Seqinfo(seqnames = paste("chr", c(2L, 4:5), sep="")),
-            elementMetadata = DataFrame(score = 1:13, GC = seq(0, 1, length=13))))
+            score = 1:13, GC = seq(0, 1, length=13))
+    )
 }
 
 test_GRangesList_construction <- function() {
@@ -55,13 +54,13 @@ test_GRangesList_coercion <- function() {
               score = 12:13)
     grl <- GRangesList(a = gr1, b = gr2)
     df <-
-      data.frame(element = rep(c("a","b"), c(3, 2)),
+      data.frame(group = togroup(grl),
+                 group_name = rep(c("a","b"), c(3, 2)),
                  seqnames = factor(c(1,1,2,"chr1","chr2")),
                  start = c(1:3,1:2), end = c(4:6,1:2),
                  width = c(4L, 4L, 4L, 1L, 1L),
                  strand = strand(c("+", "-", "*", "*", "*")),
                  score = c(10L,2L,NA,12:13),
-                 row.names = c(head(letters,3), tail(letters,2)),
                  stringsAsFactors = FALSE)
     checkIdentical(as.data.frame(grl), df)
 }
