@@ -13,16 +13,7 @@ SEXP tlocs2rlocs(
 	SEXP exonStarts,
 	SEXP exonEnds,
 	SEXP strand,
-	SEXP decreasing_rank_on_minus_strand
-);
-
-SEXP extract_transcripts(
-	SEXP classname,
-	SEXP x,
-	SEXP exonStarts,
-	SEXP exonEnds,
-	SEXP strand,
 	SEXP decreasing_rank_on_minus_strand,
-	SEXP lkup
+	SEXP error_if_out_of_bounds 
 );
 
